#!/bin/bash

# SPDX-License-Identifier: Apache-2.0

hostname=gitea-example
set -e
export FORMAT=YAML

detee-cli vm deploy --public-ip --location Random --hostname $hostname > vm_info.tmp
vm_uuid=$(grep 'uuid:' vm_info.tmp | awk '{ print $2 }')
vm_ip=$(grep 'ip:' vm_info.tmp | awk '{ print $2 }')
key_path=$(grep 'key_path:' vm_info.tmp | awk '{ print $2 }')
ssh_command="ssh -i ${key_path} -p 22 root@${vm_ip}"

echo VM created. Sleeping 10 seconds to allow SSH to start.
sleep 10
echo Installing services... Saving log to vm_log.tmp
{
  $ssh_command pacman -Syu --noconfirm
  $ssh_command pacman -S --noconfirm gitea
  $ssh_command systemctl start gitea
} > vm_log.tmp 2>&1

echo
echo "To access gitea, visit http://${vm_ip}:3000/"
echo "For testing, use the database type SQLite. Also, don't forget to set the password for the admin account."
echo
echo "To SSH into the VM, run:"
echo "    detee-cli vm ssh ${vm_uuid}"
echo "The VM will be up for 1 hour (using your LPs). Delete it by running:"
echo "    detee-cli vm delete ${vm_uuid}"
